/*
 * David Grimshaw @ Ryerson University
 * From John Hunt, Learn Java Beans Fast, (Springer 1999)
 * Counts repeatedly from minValue to maxValue if rollOver is true.
 * Otherwise, firers a MaxValueEvent.
 */

package counter;

import java.awt.*;
import java.util.*;
import java.io.Serializable;
import counter.*;

public class Counter extends Panel implements Serializable {
    private long count;
    private long initialValue = 0, maxValue = 5;
    private Label label;
    private Vector listeners = new Vector();
    private boolean rollOver;

    public Counter() {
        setBackground(Color.blue);
        setForeground(Color.white);
        setRollOver(true);
        label = new Label(initialValue + "");
        add(label);
    }

    public void  setInitialValue(long start) {
        initialValue = start;
        reset();
    }
    public long getInitialValue() {
        return initialValue;
    }

    public void  setMaxValue(long end) {
        maxValue = end;
    }
    public long getMaxValue() {
        return maxValue;
    }

    public boolean isRollOver() {
        return rollOver;
    }
    public void setRollOver(boolean state) {
        rollOver = state;
    }

    public Dimension getMinimumSize() {
        return new Dimension(30,30);
    }

    public Dimension getPreferredSize() {
        return new Dimension(60, 30);
    }

    public void reset() {
        count = initialValue;
        label.setText(count + "");
    }

    /*
     * These add and remove methods must follow the proper design patter
     * in order for introspection to take place.
     * add<event name>Listener(<event name>Listener <listner reference>)
     * remove<event name>Listener(<event name>Listener <listener reference>)
     */
    public synchronized void addMaxValueListener(MaxValueListener hearer) {
        listeners.addElement(hearer);
    }

    public synchronized void removeMaxValueListener(MaxValueListener hearer) {
        listeners.removeElement(hearer);
    }

    /*
     * Fire the event when the count reaches the trigger value
     */
     public void increment() {
         if(count != maxValue) {
             count++;
             label.setText("" + count);
         } else if (isRollOver() ) {
             reset();
         } else {
             MaxValueEvent me = new MaxValueEvent(this);
             synchronized(this) {
                 for(Enumeration e = listeners.elements();
                     e.hasMoreElements();) {
                     MaxValueListener ml = (MaxValueListener) e.nextElement();
                     ml.maxValueReached(me);
                 }
             }
         }
     }
 }
