import java.util.Hashtable;
import java.io.Serializable;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.awt.Component;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.applet.Applet;
import java.lang.reflect.Method;
import java.beans.Beans;
import sunw.beanbox.AppletSupport;
import sunw.beanbox.PropertyHookup;

public class MyApplet extends Applet implements Serializable {

    // Public 0-argument constructor

    public MyApplet() {
        InputStream is = this.getClass().getResourceAsStream("MyAppletData");
        if (is == null) {
            System.err.println("Could not locate the Applet data at MyAppletData");
            throw new Error("Could not locate the Applet data at MyAppletData");
        }
        this.setLayout(null);
        try {
            ObjectInputStream ois = new ObjectInputStream(is);
            initContentsFromStream(ois);
        } catch (Exception ex) {
            System.err.println("trouble reading from serialized data: "+ex);
            throw new Error("trouble reading from serialized data: "+ex);
        }
    }

    // Preferred size
    public Dimension getPreferredSize() {
        return getMinimumSize();
    }

    // Preferred size
    public Dimension getMinimumSize() {
        return new Dimension(395, 275);
    }

    // Read hidden-state beans from stream
    private void initContentsFromStream(java.io.ObjectInputStream ois)
        throws java.lang.ClassNotFoundException,
        java.io.IOException
    {
        Object[] data = (Object[]) ois.readObject();

        String id = (String) data[0];
        if (! id.equals("MyApplet")) {
            throw new Error("Wrong data!");
        }

        // Get references to hidden-state beans
        explicitButton1 = (sunw.demo.buttons.ExplicitButton) data[1];
        explicitButton2 = (sunw.demo.buttons.ExplicitButton) data[2];

        // Initialize the remainder of the applets contents
        // including acquiring its nested beans and reconnecting hookups.
        initContents();
    }


    // Initialize nested beans
    private void initContents()
            throws java.lang.ClassNotFoundException,
            java.io.IOException
    {
        myLoader = this.getClass().getClassLoader();
        propInstances = new Hashtable();


        // Create nested beans
        juggler1 = (sunw.demo.juggler.Juggler) Beans.instantiate(myLoader,"sunw.demo.juggler.Juggler");
        juggler1.setDebug(false);
        juggler1.setName("panel1");
        juggler1.setAnimationRate(125);

        // position all nested beans - we don't have it initially
        acquire(explicitButton1, new Rectangle(224, 39, 138, 51));
        acquire(explicitButton2, new Rectangle(228, 117, 128, 54));
        acquire(juggler1, new Rectangle(26, 53, 180, 193));

        // Add their connections
        addConnections();
    }

    private void addConnections() {
        try {
            hookup0 = new tmp.sunw.beanbox.___Hookup_1774e31c17();
            hookup0.setTarget(juggler1);
            explicitButton1.addActionListener(hookup0);
            //
            hookup1 = new tmp.sunw.beanbox.___Hookup_1774e2e786();
            hookup1.setTarget(juggler1);
            explicitButton2.addActionListener(hookup1);
            //
            //
            //   No property adaptors.  A typical example is:
            //     hookup0 = addPropertyTarget(
            //       ourButton1,
            //       "foreground",
            //       "background",
            //       "setBackground", new String[] {"java.awt.Font"});
        } catch (Exception ex) {
            System.err.println("Problems adding a target: "+ex);
            ex.printStackTrace();
        }
    }

    private void addReconnections() {
        try {
            //
            //
            //
            //   No property adaptors.  A typical example is:
            //     hookup0 = addPropertyTarget(
            //       ourButton1,
            //       "foreground",
            //       "background",
            //       "setBackground", new String[] {"java.awt.Font"});
            //   No method adaptors.  A typical example is:
            //     hookup0 = new MyActionAdaptor();
            //     ourButton1.addActionListener(hookup0);
        } catch (Exception ex) {
            System.err.println("Problems adding a target: "+ex);
            ex.printStackTrace();
        }
    }

    // Serialization code - readObject
    private void readObject(java.io.ObjectInputStream ois)
        throws java.lang.ClassNotFoundException,
        java.io.IOException
    {
        // Initialize object from stream
        Object[] data = (Object[]) ois.readObject();

        myLoader = this.getClass().getClassLoader();
        propInstances = new Hashtable();

        String id = (String) data[0];
        if (! id.equals("MyApplet")) {
            throw new Error("Wrong data!");
        }

        // Get references to nested beans
        explicitButton1 = (sunw.demo.buttons.ExplicitButton) data[1];
        explicitButton2 = (sunw.demo.buttons.ExplicitButton) data[2];
        juggler1 = (sunw.demo.juggler.Juggler) data[3];
        
        // Don't position nested beans
        acquire(explicitButton1, null);
        acquire(explicitButton2, null);
        acquire(juggler1, null);

        // Reconnect their connections
        addReconnections();
    }

    // Serialization code - writeObject
    private void writeObject(java.io.ObjectOutputStream oos)
        throws java.io.IOException
    {
        Object data[] = new Object[4];
        data[0] = "MyApplet";
        data[1] = explicitButton1;
        data[2] = explicitButton2;
        data[3] = juggler1;

        // Write the object out
        oos.writeObject(data);
    }


    // The fields used to hold the beans
    private sunw.demo.buttons.ExplicitButton explicitButton1;
    private sunw.demo.buttons.ExplicitButton explicitButton2;
    private sunw.demo.juggler.Juggler juggler1;

    // The hookups
    private tmp.sunw.beanbox.___Hookup_1774e31c17 hookup0;
    private tmp.sunw.beanbox.___Hookup_1774e2e786 hookup1;
    //   No property adaptors.  A typical example is:
    //     private PropertyHookup hookup1;

    // the loader so we can locate the resource file
    private ClassLoader myLoader;

    // =========================================== 
    // Support code
    // =========================================== 
    // It really belongs in support.jar but it is here for your reading pleasure

    // Acquire a bean
    private void acquire(Object bean, Rectangle boundsData) {
        if (!(bean instanceof Component)) {
            return;
        }
        if (bean instanceof Applet) {
            AppletSupport.assignStub((Applet) bean,
                myLoader,
            bean.getClass());
        }

        add((Component) bean);
        if (boundsData != null) {
            ((Component) bean).setBounds(boundsData);
        }
        ((Component)bean).invalidate();	// not needed?

        if (bean instanceof Applet) {
            // Start the Applet
            ((Applet)bean).start();
        }
    }

    // Add a property bound via an adaptor
    Hashtable propInstances = new Hashtable();

    private PropertyHookup addPropertyTarget(Object source,
        String propertyName,
        Object targetObject,
        String setterName, String[] setterTypeNames) throws Exception
    {
        Object args[] = new Object[1]; // arguments
        Class types[] = new Class[1]; // types
        Class pClass = java.beans.PropertyChangeListener.class;

        PropertyHookup hook = (PropertyHookup) propInstances.get(source);
        if (hook == null) {
            // This is the first property hookup on this source object
            // Push a PropertyHookup adaptor onto the source
            hook = new PropertyHookup(source);
            propInstances.put(source, hook);

            // find the adder
            types[0] = pClass;
            Method adder = source.getClass().getMethod("addPropertyChangeListener", types);
            // invoke the adder
            args[0] = hook;
            adder.invoke(source, args);
        }

        // get setter
        Method setter = targetObject.getClass().getMethod(setterName,
            getClassFromTypes(setterTypeNames));

        hook.attach(propertyName, targetObject, setter);
        return hook;
    }

    private Class[] getClassFromTypes(String types[]) throws Exception {
        Class[] back = new Class[types.length];
        for (int i=0; i<back.length; i++) {
            Class c = unwrapPrimitiveStringToClass(types[i]);

            if (c == null)
                back[i] = myLoader.loadClass(types[i]);
            else
                back[i] = c;
        }
        return back;
    }
    private Class unwrapPrimitiveStringToClass(String s) {
        if (s.equals(Byte.TYPE.getName()))        return byte.class;
        if (s.equals(Short.TYPE.getName()))       return short.class;
        if (s.equals(Integer.TYPE.getName()))     return int.class;
        if (s.equals(Long.TYPE.getName()))        return long.class;
        if (s.equals(Double.TYPE.getName()))      return double.class;
        if (s.equals(Float.TYPE.getName()))       return float.class;
        if (s.equals(Character.TYPE.getName()))   return char.class;
        if (s.equals(Boolean.TYPE.getName()))     return boolean.class;
        if (s.equals(Void.TYPE.getName()))        return void.class;
        return null;
    }
    // =========================================== 
    // End of Support code
    // =========================================== 
}
